import sys
from setuptools import setup, find_packages
import os

CUDA_MODULE = "./hastl/build_stl.py:build_stl_cuda"
OPENCL_MODULE = "./hastl/build_stl.py:build_stl_opencl"
C_MODULE = "./hastl/build_stl.py:build_stl_c"

def gen_setup(cffi_mods):
    return setup(
        name="hastl",
        version="0.1",
        author="Dmitry Serykh",
        author_email="dmitry.serykh@gmail.com",
        description=("A fast GPU implementation of STL decomposition with missing values"),
        long_description=open("README.rst", "rt").read(),
        url="https://github.com/mortvest/hastl",
        license="MIT",
        packages=find_packages(),
        install_requires=[
            "futhark-ffi>=0.13.0",
        ],
        setup_requires=[
            "futhark-ffi>=0.13.0"
        ],
        cffi_modules=cffi_mods
    )

# TODO: REWRITE THIS
try:
    gen_setup([
        CUDA_MODULE,
        OPENCL_MODULE,
        C_MODULE
    ])
except:
    print("Could not locate a working CUDA installation, skipping..")
    try:
        gen_setup([
            OPENCL_MODULE,
            C_MODULE
        ])
    except:
        print("Could not locate a working OPENCL installation, skipping..")
        gen_setup([
            C_MODULE
        ])



import re
import sys
import os

from cffi import FFI


def strip_includes(header):
    return re.sub('^(#ifdef __cplusplus\n.*\n#endif|#.*)\n', '', header, flags=re.M)

def build(source_name, out_name):
    ffibuilder = FFI()

    header_file = source_name + '.h'
    source_file = source_name + '.c'

    search = re.search('#define FUTHARK_BACKEND_([a-z0-9_]*)', open(header_file).read())
    if not search:
        sys.exit('Cannot determine Futhark backend from {}'.format(header_file))

    backend=search.group(1)

    with open(source_file) as source:
        libraries = ['m']
        extra_compile_args = ['-std=c99']
        if backend == 'opencl':
            if sys.platform == 'darwin':
                extra_compile_args += ['-framework', 'OpenCL']
            libraries += ['OpenCL']
        elif backend == 'cuda':
            libraries += ['cuda', 'cudart', 'nvrtc']
        elif backend == 'multicore':
            extra_compile_args += ['-pthread']
        ffibuilder.set_source(out_name,
                              source.read(),
                              libraries=libraries,
                              extra_compile_args=extra_compile_args)

    with open(header_file) as header:
        cdef = 'typedef void* cl_command_queue;'
        cdef += '\ntypedef void* cl_mem;'
        cdef += '\ntypedef void* CUdeviceptr;'
        cdef += strip_includes(header.read())
        cdef += "\nvoid free(void *ptr);"
        ffibuilder.cdef(cdef)

    return ffibuilder

build_stl_c = build("hastl/src/stl_c", "hastl._stl_c")
build_stl_opencl = build("hastl/src/stl_opencl", "hastl._stl_opencl")
build_stl_cuda = build("hastl/src/stl_cuda", "hastl._stl_cuda")


if __name__ == "__main__":
    build_stl_c.compile()
    build_stl_opencl.compile()
    build_stl_cuda.compile()
